import sys
from typing import Final, TypedDict, overload
from typing_extensions import LiteralString, NotRequired

if sys.platform == "win32":
    import winreg as winreg
    from os import environ as environ
else:
    class winreg:
        HKEY_USERS: Final[None]
        HKEY_CURRENT_USER: Final[None]
        HKEY_LOCAL_MACHINE: Final[None]
        HKEY_CLASSES_ROOT: Final[None]

    environ: dict[str, str]

class PlatformInfo:
    current_cpu: Final[str]

    arch: str

    def __init__(self, arch: str) -> None: ...
    @property
    def target_cpu(self) -> str: ...
    def target_is_x86(self) -> bool: ...
    def current_is_x86(self) -> bool: ...
    def current_dir(self, hidex86: bool = False, x64: bool = False) -> str: ...
    def target_dir(self, hidex86: bool = False, x64: bool = False) -> str: ...
    def cross_dir(self, forcex86: bool = False) -> str: ...

class RegistryInfo:
    if sys.platform == "win32":
        HKEYS: Final[tuple[int, int, int, int]]
    else:
        HKEYS: Final[tuple[None, None, None, None]]

    pi: PlatformInfo

    def __init__(self, platform_info: PlatformInfo) -> None: ...
    @property
    def visualstudio(self) -> LiteralString: ...
    @property
    def sxs(self) -> LiteralString: ...
    @property
    def vc(self) -> LiteralString: ...
    @property
    def vs(self) -> LiteralString: ...
    @property
    def vc_for_python(self) -> LiteralString: ...
    @property
    def microsoft_sdk(self) -> LiteralString: ...
    @property
    def windows_sdk(self) -> LiteralString: ...
    @property
    def netfx_sdk(self) -> LiteralString: ...
    @property
    def windows_kits_roots(self) -> LiteralString: ...
    @overload
    def microsoft(self, key: LiteralString, x86: bool = False) -> LiteralString: ...
    @overload
    def microsoft(self, key: str, x86: bool = False) -> str: ...  # type: ignore[misc]
    def lookup(self, key: str, name: str) -> str | None: ...

class SystemInfo:
    WinDir: Final[str]
    ProgramFiles: Final[str]
    ProgramFilesx86: Final[str]

    ri: RegistryInfo
    pi: PlatformInfo
    known_vs_paths: dict[float, str]
    vs_ver: float
    vc_ver: float

    def __init__(self, registry_info: RegistryInfo, vc_ver: float | None = None) -> None: ...
    def find_reg_vs_vers(self) -> list[float]: ...
    def find_programdata_vs_vers(self) -> dict[float, str]: ...
    @property
    def VSInstallDir(self) -> str: ...
    @property
    def VCInstallDir(self) -> str: ...
    @property
    def WindowsSdkVersion(self) -> tuple[LiteralString, ...]: ...
    @property
    def WindowsSdkLastVersion(self) -> str: ...
    @property
    def WindowsSdkDir(self) -> str | None: ...
    @property
    def WindowsSDKExecutablePath(self) -> str | None: ...
    @property
    def FSharpInstallDir(self) -> str: ...
    @property
    def UniversalCRTSdkDir(self) -> str | None: ...
    @property
    def UniversalCRTSdkLastVersion(self) -> str: ...
    @property
    def NetFxSdkVersion(self) -> tuple[LiteralString, ...]: ...
    @property
    def NetFxSdkDir(self) -> str: ...
    @property
    def FrameworkDir32(self) -> str: ...
    @property
    def FrameworkDir64(self) -> str: ...
    @property
    def FrameworkVersion32(self) -> tuple[str, ...]: ...
    @property
    def FrameworkVersion64(self) -> tuple[str, ...]: ...

class _EnvironmentDict(TypedDict):
    include: str
    lib: str
    libpath: str
    path: str
    py_vcruntime_redist: NotRequired[str | None]

class EnvironmentInfo:
    pi: PlatformInfo
    ri: RegistryInfo
    si: SystemInfo

    def __init__(self, arch: str, vc_ver: float | None = None, vc_min_ver: float = 0) -> None: ...
    @property
    def vs_ver(self) -> float: ...
    @property
    def vc_ver(self) -> float: ...
    @property
    def VSTools(self) -> list[str]: ...
    @property
    def VCIncludes(self) -> list[str]: ...
    @property
    def VCLibraries(self) -> list[str]: ...
    @property
    def VCStoreRefs(self) -> list[str]: ...
    @property
    def VCTools(self) -> list[str]: ...
    @property
    def OSLibraries(self) -> list[str]: ...
    @property
    def OSIncludes(self) -> list[str]: ...
    @property
    def OSLibpath(self) -> list[str]: ...
    @property
    def SdkTools(self) -> list[str]: ...
    @property
    def SdkSetup(self) -> list[str]: ...
    @property
    def FxTools(self) -> list[str]: ...
    @property
    def NetFxSDKLibraries(self) -> list[str]: ...
    @property
    def NetFxSDKIncludes(self) -> list[str]: ...
    @property
    def VsTDb(self) -> list[str]: ...
    @property
    def MSBuild(self) -> list[str]: ...
    @property
    def HTMLHelpWorkshop(self) -> list[str]: ...
    @property
    def UCRTLibraries(self) -> list[str]: ...
    @property
    def UCRTIncludes(self) -> list[str]: ...
    @property
    def FSharp(self) -> list[str]: ...
    @property
    def VCRuntimeRedist(self) -> str | None: ...
    def return_env(self, exists: bool = True) -> _EnvironmentDict: ...



from abc import abstractmethod

from setuptools import Command

class easy_install(Command):
    @abstractmethod
    def initialize_options(self) -> None: ...
    @abstractmethod
    def finalize_options(self) -> None: ...
    @abstractmethod
    def run(self) -> None: ...



from _typeshed import StrPath, Unused
from collections.abc import Iterable
from re import Pattern
from typing import Literal, overload

# class is entirely undocumented
class FileList:
    allfiles: Iterable[str] | None
    files: list[str]
    def __init__(self, warn: Unused = None, debug_print: Unused = None) -> None: ...
    def set_allfiles(self, allfiles: Iterable[str]) -> None: ...
    def findall(self, dir: StrPath = ".") -> None: ...
    def debug_print(self, msg: object) -> None: ...
    def append(self, item: str) -> None: ...
    def extend(self, items: Iterable[str]) -> None: ...
    def sort(self) -> None: ...
    def remove_duplicates(self) -> None: ...
    def process_template_line(self, line: str) -> None: ...
    @overload
    def include_pattern(
        self, pattern: str, anchor: bool = True, prefix: str | None = None, is_regex: Literal[False] = False
    ) -> bool: ...
    @overload
    def include_pattern(
        self, pattern: str | Pattern[str], anchor: bool = True, prefix: str | None = None, *, is_regex: Literal[True]
    ) -> bool: ...
    @overload
    def include_pattern(self, pattern: str | Pattern[str], anchor: bool, prefix: str | None, is_regex: Literal[True]) -> bool: ...
    @overload
    def exclude_pattern(
        self, pattern: str, anchor: bool = True, prefix: str | None = None, is_regex: Literal[False] = False
    ) -> bool: ...
    @overload
    def exclude_pattern(
        self, pattern: str | Pattern[str], anchor: bool = True, prefix: str | None = None, *, is_regex: Literal[True]
    ) -> bool: ...
    @overload
    def exclude_pattern(self, pattern: str | Pattern[str], anchor: bool, prefix: str | None, is_regex: Literal[True]) -> bool: ...



from _typeshed import Incomplete, StrPath
from abc import abstractmethod
from collections.abc import ItemsView, Iterable, Mapping, Sequence
from typing import Any, Literal, Protocol, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import Never, NotRequired

from ._distutils.cmd import Command as _Command
from ._distutils.dist import Distribution as _Distribution
from ._distutils.extension import Extension as _Extension
from .command.alias import alias
from .command.bdist_egg import bdist_egg
from .command.bdist_rpm import bdist_rpm
from .command.bdist_wheel import bdist_wheel
from .command.build import build
from .command.build_clib import build_clib
from .command.build_ext import build_ext
from .command.build_py import build_py
from .command.develop import develop
from .command.dist_info import dist_info
from .command.easy_install import easy_install
from .command.editable_wheel import editable_wheel
from .command.egg_info import egg_info
from .command.install import install
from .command.install_egg_info import install_egg_info
from .command.install_lib import install_lib
from .command.install_scripts import install_scripts
from .command.rotate import rotate
from .command.saveopts import saveopts
from .command.sdist import sdist
from .command.setopt import setopt
from .depends import Require as Require
from .discovery import _Finder
from .dist import Distribution as Distribution
from .extension import Extension as Extension
from .warnings import SetuptoolsDeprecationWarning as SetuptoolsDeprecationWarning

_CommandT = TypeVar("_CommandT", bound=_Command)
_DistributionT = TypeVar("_DistributionT", bound=_Distribution, default=Distribution)
_KT = TypeVar("_KT")
_VT_co = TypeVar("_VT_co", covariant=True)

__all__ = [
    "setup",
    "Distribution",
    "Command",
    "Extension",
    "Require",
    "SetuptoolsDeprecationWarning",
    "find_packages",
    "find_namespace_packages",
]

__version__: str

# We need any Command subclass to be valid
# Any: pyright would accept using covariance in __setitem__, but mypy won't let a dict be assignable to this protocol
# This is unsound, but it's a quirk of setuptools' internals
@type_check_only
class _DictLike(Protocol[_KT, _VT_co]):
    # See note about using _VT_co instead of Any
    def get(self, key: _KT, default: Any | None = None, /) -> _VT_co | None: ...
    def items(self) -> ItemsView[_KT, _VT_co]: ...
    def keys(self) -> Iterable[_KT]: ...
    def __getitem__(self, key: _KT, /) -> _VT_co: ...
    def __contains__(self, x: object, /) -> bool: ...

@type_check_only
class _MutableDictLike(_DictLike[_KT, _VT_co], Protocol):
    # See note about using _VT_co instead of Any
    def __setitem__(self, key: _KT, value: Any, /) -> None: ...
    def setdefault(self, key: _KT, default: Any, /) -> _VT_co: ...

@type_check_only
class _BuildInfo(TypedDict):
    sources: list[str] | tuple[str, ...]
    obj_deps: NotRequired[dict[str, list[str] | tuple[str, ...]]]
    macros: NotRequired[list[tuple[str] | tuple[str, str | None]]]
    include_dirs: NotRequired[list[str]]
    cflags: NotRequired[list[str]]

find_packages = _Finder.find
find_namespace_packages = _Finder.find

def setup(
    *,
    # Attributes from distutils.dist.DistributionMetadata.set_*
    # These take priority over attributes from distutils.dist.DistributionMetadata.__init__
    keywords: str | Iterable[str] = ...,
    platforms: str | Iterable[str] = ...,
    classifiers: str | Iterable[str] = ...,
    requires: Iterable[str] = ...,
    provides: Iterable[str] = ...,
    obsoletes: Iterable[str] = ...,
    # Attributes from distutils.dist.DistributionMetadata.__init__
    # These take priority over attributes from distutils.dist.Distribution.__init__
    name: str | None = None,
    version: str | None = None,
    author: str | None = None,
    author_email: str | None = None,
    maintainer: str | None = None,
    maintainer_email: str | None = None,
    url: str | None = None,
    license: str | None = None,
    description: str | None = None,
    long_description: str | None = None,
    download_url: str | None = None,
    # Attributes from distutils.dist.Distribution.__init__ (except self.metadata)
    # These take priority over attributes from distutils.dist.Distribution.display_option_names
    verbose: bool = True,
    dry_run: bool = False,
    help: bool = False,
    cmdclass: _MutableDictLike[str, type[_Command]] = {},
    command_packages: str | list[str] | None = None,
    script_name: StrPath | None = ...,  # default is actually set in distutils.core.setup
    script_args: list[str] | None = ...,  # default is actually set in distutils.core.setup
    command_options: _MutableDictLike[str, _DictLike[str, tuple[str, str]]] = {},
    packages: list[str] | None = None,
    package_dir: Mapping[str, str] | None = None,
    py_modules: list[str] | None = None,
    libraries: list[tuple[str, _BuildInfo]] | None = None,
    headers: list[str] | None = None,
    ext_modules: Sequence[_Extension] | None = None,
    ext_package: str | None = None,
    include_dirs: list[str] | None = None,
    extra_path: Never = ...,  # Deprecated
    scripts: list[str] | None = None,
    data_files: list[tuple[str, Sequence[str]]] | None = None,
    password: str = "",
    command_obj: _MutableDictLike[str, _Command] = {},
    have_run: _MutableDictLike[str, bool] = {},
    # kwargs used directly in distutils.dist.Distribution.__init__
    options: Mapping[str, Mapping[str, str]] | None = None,
    licence: Never = ...,  # Deprecated
    # Attributes from distutils.dist.Distribution.display_option_names
    # (this can more easily be copied from the `if TYPE_CHECKING` block)
    help_commands: bool = False,
    fullname: str | Literal[False] = False,
    contact: str | Literal[False] = False,
    contact_email: str | Literal[False] = False,
    # kwargs used directly in setuptools.dist.Distribution.__init__
    # and attributes from setuptools.dist.Distribution.__init__
    package_data: _DictLike[str, list[str]] = {},
    dist_files: list[tuple[str, str, str]] = [],
    include_package_data: bool | None = None,
    exclude_package_data: _DictLike[str, list[str]] | None = None,
    src_root: str | None = None,
    dependency_links: list[str] = [],
    setup_requires: list[str] = [],
    # From Distribution._DISTUTILS_UNSUPPORTED_METADATA set in Distribution._set_metadata_defaults
    long_description_content_type: str | None = None,
    project_urls: _DictLike[Incomplete, Incomplete] = {},
    provides_extras: _MutableDictLike[Incomplete, Incomplete] = {},
    license_expression: str | None = None,
    license_file: Never = ...,  # Deprecated
    license_files: Iterable[str] | None = None,
    install_requires: str | Iterable[str] = [],
    extras_require: _DictLike[Incomplete, Incomplete] = {},
    # kwargs used directly in distutils.core.setup
    distclass: type[_DistributionT] = Distribution,  # type: ignore[assignment] # noqa: Y011
    # Custom Distributions could accept more params
    **attrs: Any,
) -> _DistributionT: ...

class Command(_Command):
    command_consumes_arguments: bool
    distribution: Distribution
    # Any: Dynamic command subclass attributes
    def __init__(self, dist: Distribution, **kw: Any) -> None: ...
    # Note: Commands that setuptools doesn't re-expose are considered deprecated (they must be imported from distutils directly)
    # So we're not listing them here. This list comes directly from the setuptools/command folder. Minus the test command.
    @overload  # type: ignore[override]
    def get_finalized_command(self, command: Literal["alias"], create: bool | Literal[0, 1] = 1) -> alias: ...
    @overload
    def get_finalized_command(self, command: Literal["bdist_egg"], create: bool | Literal[0, 1] = 1) -> bdist_egg: ...
    @overload
    def get_finalized_command(self, command: Literal["bdist_rpm"], create: bool | Literal[0, 1] = 1) -> bdist_rpm: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["bdist_wheel"], create: bool | Literal[0, 1] = 1) -> bdist_wheel: ...
    @overload
    def get_finalized_command(self, command: Literal["build"], create: bool | Literal[0, 1] = 1) -> build: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["build_clib"], create: bool | Literal[0, 1] = 1) -> build_clib: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["build_ext"], create: bool | Literal[0, 1] = 1) -> build_ext: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["build_py"], create: bool | Literal[0, 1] = 1) -> build_py: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["develop"], create: bool | Literal[0, 1] = 1) -> develop: ...
    @overload
    def get_finalized_command(self, command: Literal["dist_info"], create: bool | Literal[0, 1] = 1) -> dist_info: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["easy_install"], create: bool | Literal[0, 1] = 1) -> easy_install: ...
    @overload
    def get_finalized_command(self, command: Literal["editable_wheel"], create: bool | Literal[0, 1] = 1) -> editable_wheel: ...
    @overload
    def get_finalized_command(self, command: Literal["egg_info"], create: bool | Literal[0, 1] = 1) -> egg_info: ...
    @overload
    def get_finalized_command(self, command: Literal["install"], create: bool | Literal[0, 1] = 1) -> install: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(
        self, command: Literal["install_egg_info"], create: bool | Literal[0, 1] = 1
    ) -> install_egg_info: ...
    @overload
    def get_finalized_command(self, command: Literal["install_lib"], create: bool | Literal[0, 1] = 1) -> install_lib: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["install_scripts"], create: bool | Literal[0, 1] = 1) -> install_scripts: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["rotate"], create: bool | Literal[0, 1] = 1) -> rotate: ...
    @overload
    def get_finalized_command(self, command: Literal["saveopts"], create: bool | Literal[0, 1] = 1) -> saveopts: ...
    @overload
    def get_finalized_command(self, command: Literal["sdist"], create: bool | Literal[0, 1] = 1) -> sdist: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["setopt"], create: bool | Literal[0, 1] = 1) -> setopt: ...
    @overload
    def get_finalized_command(self, command: str, create: bool | Literal[0, 1] = 1) -> Command: ...
    @overload  # type: ignore[override] # Extra **kw param
    def reinitialize_command(self, command: Literal["alias"], reinit_subcommands: bool = False, **kw) -> alias: ...
    @overload
    def reinitialize_command(self, command: Literal["bdist_egg"], reinit_subcommands: bool = False, **kw) -> bdist_egg: ...
    @overload
    def reinitialize_command(self, command: Literal["bdist_rpm"], reinit_subcommands: bool = False, **kw) -> bdist_rpm: ...
    @overload
    def reinitialize_command(self, command: Literal["bdist_wheel"], reinit_subcommands: bool = False, **kw) -> bdist_wheel: ...
    @overload
    def reinitialize_command(self, command: Literal["build"], reinit_subcommands: bool = False, **kw) -> build: ...
    @overload
    def reinitialize_command(self, command: Literal["build_clib"], reinit_subcommands: bool = False, **kw) -> build_clib: ...
    @overload
    def reinitialize_command(self, command: Literal["build_ext"], reinit_subcommands: bool = False, **kw) -> build_ext: ...
    @overload
    def reinitialize_command(self, command: Literal["build_py"], reinit_subcommands: bool = False, **kw) -> build_py: ...
    @overload
    def reinitialize_command(self, command: Literal["develop"], reinit_subcommands: bool = False, **kw) -> develop: ...
    @overload
    def reinitialize_command(self, command: Literal["dist_info"], reinit_subcommands: bool = False, **kw) -> dist_info: ...
    @overload
    def reinitialize_command(self, command: Literal["easy_install"], reinit_subcommands: bool = False, **kw) -> easy_install: ...
    @overload
    def reinitialize_command(
        self, command: Literal["editable_wheel"], reinit_subcommands: bool = False, **kw
    ) -> editable_wheel: ...
    @overload
    def reinitialize_command(self, command: Literal["egg_info"], reinit_subcommands: bool = False, **kw) -> egg_info: ...
    @overload
    def reinitialize_command(self, command: Literal["install"], reinit_subcommands: bool = False, **kw) -> install: ...
    @overload
    def reinitialize_command(
        self, command: Literal["install_egg_info"], reinit_subcommands: bool = False, **kw
    ) -> install_egg_info: ...
    @overload
    def reinitialize_command(self, command: Literal["install_lib"], reinit_subcommands: bool = False, **kw) -> install_lib: ...
    @overload
    def reinitialize_command(
        self, command: Literal["install_scripts"], reinit_subcommands: bool = False, **kw
    ) -> install_scripts: ...
    @overload
    def reinitialize_command(self, command: Literal["rotate"], reinit_subcommands: bool = False, **kw) -> rotate: ...
    @overload
    def reinitialize_command(self, command: Literal["saveopts"], reinit_subcommands: bool = False, **kw) -> saveopts: ...
    @overload
    def reinitialize_command(self, command: Literal["sdist"], reinit_subcommands: bool = False, **kw) -> sdist: ...
    @overload
    def reinitialize_command(self, command: Literal["setopt"], reinit_subcommands: bool = False, **kw) -> setopt: ...
    @overload
    def reinitialize_command(self, command: str, reinit_subcommands: bool = False, **kw) -> Command: ...
    @overload
    def reinitialize_command(self, command: _CommandT, reinit_subcommands: bool = False, **kw) -> _CommandT: ...
    @abstractmethod
    def initialize_options(self) -> None: ...
    @abstractmethod
    def finalize_options(self) -> None: ...
    @abstractmethod
    def run(self) -> None: ...

class sic(str): ...



from _typeshed import GenericPath, Incomplete, StrPath
from collections.abc import Iterator
from types import CodeType
from typing import AnyStr, ClassVar, Final, Literal, TypeVar
from zipfile import _ZipFileMode

from .. import Command

_StrPathT = TypeVar("_StrPathT", bound=StrPath)

def strip_module(filename): ...
def sorted_walk(dir: GenericPath[AnyStr]) -> Iterator[tuple[AnyStr, list[AnyStr], list[AnyStr]]]: ...
def write_stub(resource, pyfile) -> None: ...

class bdist_egg(Command):
    description: str
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]
    bdist_dir: Incomplete
    plat_name: str
    keep_temp: bool
    dist_dir: Incomplete
    skip_build: bool
    egg_output: Incomplete
    exclude_source_files: Incomplete
    def initialize_options(self) -> None: ...
    egg_info: Incomplete
    def finalize_options(self) -> None: ...
    def do_install_data(self) -> None: ...
    def get_outputs(self): ...
    def call_command(self, cmdname, **kw): ...
    stubs: Incomplete
    def run(self) -> None: ...
    def zap_pyfiles(self) -> None: ...
    def zip_safe(self): ...
    def gen_header(self) -> Literal["w"]: ...
    def copy_metadata_to(self, target_dir) -> None: ...
    def get_ext_outputs(self): ...

NATIVE_EXTENSIONS: Final[dict[str, None]]

def walk_egg(egg_dir: StrPath) -> Iterator[tuple[str, list[str], list[str]]]: ...
def analyze_egg(egg_dir, stubs): ...
def write_safety_flag(egg_dir, safe) -> None: ...

safety_flags: Incomplete

def scan_module(egg_dir, base, name, stubs): ...
def iter_symbols(code: CodeType) -> Iterator[str]: ...
def can_scan() -> bool: ...

INSTALL_DIRECTORY_ATTRS: Final[list[str]]

def make_zipfile(
    zip_filename: _StrPathT,
    base_dir,
    verbose: bool = False,
    dry_run: bool = False,
    compress: bool = True,
    mode: _ZipFileMode = "w",
) -> _StrPathT: ...



from _typeshed import Incomplete
from typing import ClassVar

from .. import Command, namespaces

class install_egg_info(namespaces.Installer, Command):
    description: str
    user_options: ClassVar[list[tuple[str, str, str]]]
    install_dir: Incomplete
    def initialize_options(self) -> None: ...
    source: Incomplete
    target: str
    outputs: list[str]
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def get_outputs(self): ...
    def copytree(self): ...



from _typeshed import Incomplete, Unused
from collections.abc import Callable
from typing import ClassVar

from ..cmd import Command

def show_formats() -> None: ...

class sdist(Command):
    description: ClassVar[str]

    def checking_metadata(self): ...

    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]
    help_options: ClassVar[list[tuple[str, str | None, str, Callable[[], Unused]]]]
    negative_opt: ClassVar[dict[str, str]]
    READMES: ClassVar[tuple[str, ...]]
    template: Incomplete
    manifest: Incomplete
    use_defaults: bool
    prune: bool
    manifest_only: bool
    force_manifest: bool
    formats: Incomplete
    keep_temp: bool
    dist_dir: Incomplete
    archive_files: Incomplete
    metadata_check: int  # Soon to be updated to boolean upstream
    owner: Incomplete
    group: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    filelist: Incomplete
    def run(self) -> None: ...
    def get_file_list(self) -> None: ...
    def add_defaults(self) -> None: ...
    def read_template(self) -> None: ...
    def prune_file_list(self) -> None: ...
    def write_manifest(self) -> None: ...
    def read_manifest(self) -> None: ...
    def make_release_tree(self, base_dir, files) -> None: ...
    def make_distribution(self) -> None: ...
    def get_archive_files(self): ...

def is_comment(line: str) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import ClassVar, Final, Literal

from setuptools import Command

def safe_version(version: str) -> str: ...

setuptools_major_version: Final[int]

PY_LIMITED_API_PATTERN: Final[str]

def python_tag() -> str: ...
def get_platform(archive_root: str | None) -> str: ...
def get_flag(var: str, fallback: bool, expected: bool = True, warn: bool = True) -> bool: ...
def get_abi_tag() -> str | None: ...
def safer_version(version: str) -> str: ...

class bdist_wheel(Command):
    description: ClassVar[str]
    supported_compressions: ClassVar[dict[str, int]]
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]

    bdist_dir: str | None
    data_dir: str
    plat_name: str | None
    plat_tag: str | None
    format: str
    keep_temp: bool
    dist_dir: str | None
    egginfo_dir: str | None
    root_is_pure: bool | None
    skip_build: bool
    relative: bool
    owner: Incomplete | None
    group: Incomplete | None
    universal: bool
    compression: str | int
    python_tag: str
    build_number: str | None
    py_limited_api: str | Literal[False]
    plat_name_supplied: bool

    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    @property
    def wheel_dist_name(self) -> str: ...
    def get_tag(self) -> tuple[str, str, str]: ...
    def run(self) -> None: ...
    def write_wheelfile(self, wheelfile_base: str, generator: str = ...) -> None: ...
    @property
    def license_paths(self) -> Iterable[str]: ...
    def egg2dist(self, egginfo_path: str, distinfo_path: str) -> None: ...



from _typeshed import Incomplete, StrPath, Unused
from typing import ClassVar

from setuptools.dist import Distribution

from .._distutils.cmd import _StrPathT
from .._distutils.command import build_py as orig

def make_writable(target) -> None: ...

class build_py(orig.build_py):
    distribution: Distribution  # override distutils.dist.Distribution with setuptools.dist.Distribution
    editable_mode: ClassVar[bool]
    package_data: dict[str, list[str]]
    exclude_package_data: dict[Incomplete, Incomplete]
    def finalize_options(self) -> None: ...
    def copy_file(  # type: ignore[override] # No overload, str support only
        self,
        infile: StrPath,
        outfile: _StrPathT,
        preserve_mode: bool = True,
        preserve_times: bool = True,
        link: str | None = None,
        level: Unused = 1,
    ) -> tuple[_StrPathT | str, bool]: ...
    def run(self) -> None: ...
    data_files: list[tuple[str, str, str, list[str]]]
    def __getattr__(self, attr: str): ...
    def get_data_files_without_manifest(self) -> list[tuple[str, str, str, list[str]]]: ...
    def find_data_files(self, package, src_dir) -> list[str]: ...
    def get_outputs(self, include_bytecode: bool = True) -> list[str]: ...  # type: ignore[override] # Using a real boolean instead of 0|1
    def build_package_data(self) -> None: ...
    manifest_files: dict[str, list[str]]
    def get_output_mapping(self) -> dict[str, str]: ...
    def analyze_manifest(self) -> None: ...
    def get_data_files(self) -> None: ...
    def check_package(self, package, package_dir): ...
    def initialize_options(self) -> None: ...
    packages_checked: dict[Incomplete, Incomplete]
    def get_package_dir(self, package: str) -> str: ...
    def exclude_data_files(self, package, src_dir, files): ...

def assert_relative(path): ...



from _typeshed import StrOrBytesPath, StrPath, Unused
from collections.abc import MutableSequence, Sequence
from subprocess import _ENV
from typing import Literal, overload

@overload
def spawn(
    cmd: Sequence[StrOrBytesPath],
    search_path: Literal[False],
    verbose: Unused = False,
    dry_run: bool = False,
    env: _ENV | None = None,
) -> None: ...
@overload
def spawn(
    cmd: MutableSequence[bytes | StrPath],
    search_path: Literal[True] = True,
    verbose: Unused = False,
    dry_run: bool = False,
    env: _ENV | None = None,
) -> None: ...
def find_executable(executable: str, path: str | None = None) -> str | None: ...



from .compilers.C import base
from .compilers.C.base import gen_lib_options, gen_preprocess_options, get_default_compiler, new_compiler, show_compilers
from .compilers.C.errors import CompileError, LinkError

__all__ = [
    "CompileError",
    "LinkError",
    "gen_lib_options",
    "gen_preprocess_options",
    "get_default_compiler",
    "new_compiler",
    "show_compilers",
]

CCompiler = base.Compiler



from importlib import metadata
from typing import Any
from typing_extensions import deprecated

@deprecated(
    """
    `setuptools.installer` and `fetch_build_eggs` are deprecated.
    Requirements should be satisfied by a PEP 517 installer.
    If you are using pip, you can try `pip install --use-pep517`.
    """
)
def fetch_build_egg(dist, req) -> metadata.Distribution | metadata.PathDistribution: ...

# Returns packaging.requirements.Requirement
# But since this module is deprecated, we avoid declaring a dependency on packaging
def strip_marker(req) -> Any: ...



from _typeshed import Incomplete, Unused
from collections.abc import Callable
from typing import ClassVar

from ..cmd import Command

class build_clib(Command):
    description: ClassVar[str]
    user_options: ClassVar[list[tuple[str, str, str]]]
    boolean_options: ClassVar[list[str]]
    help_options: ClassVar[list[tuple[str, str | None, str, Callable[[], Unused]]]]
    build_clib: Incomplete
    build_temp: Incomplete
    libraries: Incomplete
    include_dirs: Incomplete
    define: Incomplete
    undef: Incomplete
    debug: Incomplete
    force: bool
    compiler: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def check_library_list(self, libraries) -> None: ...
    def get_library_names(self): ...
    def get_source_files(self): ...
    def build_libraries(self, libraries) -> None: ...



from _typeshed import Incomplete, Unused
from collections.abc import Callable
from typing import ClassVar

from ..cmd import Command
from ..extension import Extension

class build_ext(Command):
    description: ClassVar[str]
    sep_by: Incomplete
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]
    help_options: ClassVar[list[tuple[str, str | None, str, Callable[[], Unused]]]]
    extensions: Incomplete
    build_lib: Incomplete
    plat_name: Incomplete
    build_temp: Incomplete
    inplace: bool
    package: Incomplete
    include_dirs: Incomplete
    define: Incomplete
    undef: Incomplete
    libraries: Incomplete
    library_dirs: Incomplete
    rpath: Incomplete
    link_objects: Incomplete
    debug: Incomplete
    force: Incomplete
    compiler: Incomplete
    swig: Incomplete
    swig_cpp: Incomplete
    swig_opts: Incomplete
    user: Incomplete
    parallel: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def check_extensions_list(self, extensions) -> None: ...
    def get_source_files(self): ...
    def get_outputs(self): ...
    def build_extensions(self) -> None: ...
    def build_extension(self, ext) -> None: ...
    def swig_sources(self, sources, extension): ...
    def find_swig(self): ...
    def get_ext_fullpath(self, ext_name: str) -> str: ...
    def get_ext_fullname(self, ext_name: str) -> str: ...
    def get_ext_filename(self, ext_name: str) -> str: ...
    def get_export_symbols(self, ext: Extension) -> list[str]: ...
    def get_libraries(self, ext: Extension) -> list[str]: ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing import Any, ClassVar

from setuptools.dist import Distribution

from .._distutils.command import install as orig

class install(orig.install):
    distribution: Distribution  # override distutils.dist.Distribution with setuptools.dist.Distribution
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]
    # Any to work around variance issues
    new_commands: ClassVar[list[tuple[str, Callable[[Any], bool]] | None]]
    old_and_unmanageable: Incomplete
    single_version_externally_managed: bool | None
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    path_file: Incomplete
    extra_dirs: str
    def handle_extra_path(self): ...



from types import FunctionType
from typing import TypeVar, overload

_T = TypeVar("_T")
_UnpatchT = TypeVar("_UnpatchT", type, FunctionType)
__all__: list[str] = []

@overload
def get_unpatched(item: _UnpatchT) -> _UnpatchT: ...  # type: ignore[overload-overlap]
@overload
def get_unpatched(item: object) -> None: ...
def get_unpatched_class(cls: type[_T]) -> type[_T]: ...
def patch_all() -> None: ...
def patch_func(replacement, target_mod, func_name) -> None: ...
def get_unpatched_function(candidate): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..cmd import Command

class install_data(Command):
    description: ClassVar[str]
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: Incomplete
    install_dir: Incomplete
    outfiles: Incomplete
    root: Incomplete
    force: bool
    data_files: Incomplete
    warn_dir: bool
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def get_inputs(self): ...
    def get_outputs(self): ...



from _typeshed import StrPath
from collections.abc import Sequence
from typing import ClassVar, Final

from . import base

PLAT_SPEC_TO_RUNTIME: Final[dict[str, str]]

class Compiler(base.Compiler):
    src_extensions: ClassVar[list[str]]
    res_extension: ClassVar[str]
    obj_extension: ClassVar[str]
    static_lib_extension: ClassVar[str]
    shared_lib_extension: ClassVar[str]
    shared_lib_format: ClassVar[str]
    static_lib_format = shared_lib_format
    exe_extension: ClassVar[str]
    initialized: bool
    def initialize(self, plat_name: str | None = None) -> None: ...
    @property
    def out_extensions(self) -> dict[str, str]: ...
    def spawn(self, cmd: Sequence[bytes | StrPath]): ...  # type: ignore[override] # Less params



from ._distutils import errors as _distutils_errors

ByteCompileError = _distutils_errors.DistutilsByteCompileError
CCompilerError = _distutils_errors.CCompilerError
ClassError = _distutils_errors.DistutilsClassError
CompileError = _distutils_errors.CompileError
ExecError = _distutils_errors.DistutilsExecError
FileError = _distutils_errors.DistutilsFileError
InternalError = _distutils_errors.DistutilsInternalError
LibError = _distutils_errors.LibError
LinkError = _distutils_errors.LinkError
ModuleError = _distutils_errors.DistutilsModuleError
OptionError = _distutils_errors.DistutilsOptionError
PlatformError = _distutils_errors.DistutilsPlatformError
PreprocessError = _distutils_errors.PreprocessError
SetupError = _distutils_errors.DistutilsSetupError
TemplateError = _distutils_errors.DistutilsTemplateError
UnknownFileError = _distutils_errors.UnknownFileError
BaseError = _distutils_errors.DistutilsError

class InvalidConfigError(OptionError): ...
class RemovedConfigError(OptionError): ...
class RemovedCommandError(BaseError, RuntimeError): ...
class PackageDiscoveryError(BaseError, RuntimeError): ...



from setuptools._distutils.ccompiler import *
from setuptools._distutils.ccompiler import CCompiler as CCompiler

__all__ = [
    "CompileError",
    "LinkError",
    "gen_lib_options",
    "gen_preprocess_options",
    "get_default_compiler",
    "new_compiler",
    "show_compilers",
]



from typing import IO, Any, Literal, TypeVar

_T = TypeVar("_T")

__all__ = ["Require", "find_module", "get_module_constant", "extract_constant"]

def find_module(
    module, paths=None
) -> tuple[IO[Any], str | None, tuple[str, Literal["", "r", "rb"], Literal[7, 6, 1, 2, 3, -1]]]: ...

class Require:
    def __init__(self, name, requested_version, module, homepage: str = "", attribute=None, format=None) -> None: ...
    def full_name(self): ...
    def version_ok(self, version): ...
    def get_version(self, paths=None, default: _T | Literal["unknown"] = "unknown") -> _T | Literal["unknown"] | None | Any: ...
    def is_present(self, paths=None): ...
    def is_current(self, paths=None): ...

def get_module_constant(module, symbol, default: _T | int = -1, paths=None) -> _T | int | None | Any: ...
def extract_constant(code, symbol, default: _T | int = -1) -> _T | int | None | Any: ...



from _typeshed import GenericPath, StrPath, Unused
from collections.abc import Callable, Iterable, Mapping
from typing import AnyStr, Literal
from typing_extensions import TypeVarTuple, Unpack

_Ts = TypeVarTuple("_Ts")

def get_host_platform() -> str: ...
def get_platform() -> str: ...
def get_macosx_target_ver_from_syscfg(): ...
def get_macosx_target_ver(): ...
def split_version(s: str) -> list[int]: ...
def convert_path(pathname: StrPath) -> str: ...
def change_root(new_root: GenericPath[AnyStr], pathname: GenericPath[AnyStr]) -> AnyStr: ...
def check_environ() -> None: ...
def subst_vars(s: str, local_vars: Mapping[str, object]) -> str: ...
def grok_environment_error(exc: object, prefix: str = "error: ") -> str: ...
def split_quoted(s: str) -> list[str]: ...
def execute(
    func: Callable[[Unpack[_Ts]], Unused],
    args: tuple[Unpack[_Ts]],
    msg: str | None = None,
    verbose: bool = False,
    dry_run: bool = False,
) -> None: ...
def strtobool(val: str) -> Literal[0, 1]: ...
def byte_compile(
    py_files: Iterable[str],
    optimize: int = 0,
    force: bool = False,
    prefix: str | None = None,
    base_dir: str | None = None,
    verbose: bool = True,
    dry_run: bool = False,
    direct: bool | None = None,
) -> None: ...
def rfc822_escape(header: str) -> str: ...
def is_mingw() -> bool: ...



from setuptools.dist import Distribution

from .._distutils.command import bdist_rpm as orig

class bdist_rpm(orig.bdist_rpm):
    distribution: Distribution  # override distutils.dist.Distribution with setuptools.dist.Distribution
    def run(self) -> None: ...
